@@ -11,7 +11,7 @@ our $VERSION;
 
 BEGIN {
     use XSLoader;
-    $VERSION = '0.03';
+    $VERSION = '0.04';
     XSLoader::load('Math::Geometry::Delaunay');
     }
 
@@ -102,7 +102,7 @@ sub verbose { # 0 to 3
     return $_[0]->{V};
     }
 
-# everything to add input geometery
+# everything to add input geometry
 
 sub addRegion {
     my $self = shift;
@@ -433,8 +433,8 @@ sub topology {
     my @nodes = map {{ point => $_, attributes => [], marker => undef, elements => [], edges => [] , segments => []}} ltolol(2,$triio->pointlist);
     my @eles  = map {my $ele={ nodes=>[map {$nodes[$_]} @{$_}],edges => [], neighbors => [], marker => undef, attributes => [] };map {push @{$_->{elements}},$ele} @{$ele->{nodes}};$ele} ltolol($triio->numberofcorners,$triio->trianglelist);
     my $ecnt = 0; # The index for edges will be the link between Delaunay and Voronoi topology pairs.
-    my @edges = map {my $edg={ nodes=>[map {$nodes[$_]} @{$_}],marker => undef, elements => [], vector => undef, index => $ecnt++};map {push @{$_->{edges}   },$edg} @{$edg->{nodes}};$isVoronoi = 1 if ($_->[0] == -1 || $_->[1] == -1);$edg} ltolol(2,$triio->edgelist);
-    my @segs  = map {my $edg={ nodes=>[map {$nodes[$_]} @{$_}],marker => undef, elements => [] };map {push @{$_->{segments}},$edg} @{$edg->{nodes}};$edg} ltolol(2,$triio->segmentlist);
+    my @edges = map {my $edg={ nodes=>[map {$nodes[$_]} grep {$_>-1} @{$_}],marker => undef, elements => [], vector => undef, index => $ecnt++};foreach (@{$edg->{nodes}}) {push @{$_->{edges}   },$edg};$isVoronoi = 1 if ($_->[0] == -1 || $_->[1] == -1);$edg} ltolol(2,$triio->edgelist);
+    my @segs  = map {my $edg={ nodes=>[map {$nodes[$_]}              @{$_}],marker => undef, elements => []                                   };foreach (@{$edg->{nodes}}) {push @{$_->{segments}},$edg};                                                   $edg} ltolol(2,$triio->segmentlist);
  
     my @elementattributes;
     if ($triio->numberoftriangleattributes) {
@@ -458,7 +458,7 @@ sub topology {
     for (my $i=0;$i<@edgemarkers;$i++) {
         $edges[$i]->{marker} = $edgemarkers[$i];
         }
-    my @segmentmarkers = $triio->segmentmarkerlist; # because some can be internal to boundarys
+    my @segmentmarkers = $triio->segmentmarkerlist; # because some can be internal to boundaries
     for (my $i=0;$i<@segmentmarkers;$i++) {
         $segs[$i]->{marker} = $segmentmarkers[$i];
         }
@@ -545,7 +545,7 @@ sub get_point_in_polygon {
 
     my @intersections = grep {
         # print "interdist: ",$_->[2],"\n";
-        # Note the filting out of very-near-zero distance intersections -
+        # Note the filtering out of very-near-zero distance intersections -
         # There should always be two intersections where the base of 
         # the test ray sits on the $poly->[$bottom_left_index] point:
         # one intersection for each adjacent segment. Better to get them
@@ -677,7 +677,7 @@ Math::Geometry::Delaunay - Quality Mesh Generator and Delaunay Triangulator
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =cut
 
@@ -1021,7 +1021,7 @@ L<http://www.cs.cmu.edu/~quake/triangle.switch.html>
     my $tri = Math::Geometry::Delaunay->new(TRI_CCDT, 'q15', 'a3.5');
 
 
-Options set by switches passed to C<new()> may be overriden later by the 
+Options set by switches passed to C<new()> may be overridden later by the 
 corresponding option-setting methods, any time before C<triangulate()> 
 is invoked.
 
@@ -1073,16 +1073,16 @@ has boundary marker 0.
 The ray flag is 0 if the edge is not a ray, or 1 or 2, to indicate 
 which vertex is actually a unit vector indicating the direction of the ray.
 
-Import of the mesh data from the C data structures will be defered until
+Import of the mesh data from the C data structures will be deferred until
 actually requested from the list fetching methods above. For speed and 
-lower memory footprint, access only what you need, and consider supressing 
+lower memory footprint, access only what you need, and consider suppressing 
 output you don't need with option switches.
 
 =head3 topological output
 
 When triangulate is invoked in scalar or array context, it returns a hash ref 
 containing the cross-referenced nodes, elements, edges, and PSLG segments of the
-triangulation. In array context, with the "v" switch enabled, the Voroni
+triangulation. In array context, with the "v" switch enabled, the Voronoi
 topology is the second item returned.
 
     my $topology = $tri->triangulate();
@@ -1115,18 +1115,42 @@ topology is the second item returned.
                   {
                   nodes    => [noderef0, noderef1], # only one for a ray
                   elements => [elemref0, elemref1], # one if on boundary
-                  vector   => undefined or [x, y]   # ray direction
+                  vector   => undefined or [x, y],  # ray direction
                   marker   => 1 or 0 or undefined,  # boundary marker
+                  index    => <integer> # edge's index in edge list
                   },
                   ... more edges like that
                     
                 ],
     segments => [
-                  { same structure as edges },
+                  {
+                  nodes    => [noderef0, noderef1],
+                  elements => [elemref0, elemref1], # one if on boundary
+                  marker   => 1 or 0 or undefined   # boundary marker
+                  },
                   ... more segments
                 ]
     }
 
+=head3 cross-referencing Delaunay and Voronoi
+
+Corresponding edges in the Delaunay and Voronoi outputs have the same index
+number in their respective edge lists. 
+
+In the topological output, any edge in a triangulation has a record of its own 
+index number that can by used to look up the corresponding edge in the Voronoi 
+diagram topology, or vice versa, like so:
+
+    ($topo, $voronoi_topo) = $tri->triangulate('ev');
+    
+    # get an edge reference where it's not obvious what the edge's index is
+    
+    $delaunay_edge = $topo->{nodes}->[-1]->{edges}->[-1];
+    
+    # this gets a reference to the corresponding edge in the Voronoi diagram
+    
+    $voronoi_edge = $voronoi_topo->{edges}->[$delaunay_edge->{index}];
+
 =head1 METHODS TO SET SOME Triangle OPTIONS
 
 =head2 area_constraint
@@ -1147,7 +1171,7 @@ With one argument, sets the minimum angle allowed for triangles added in the
 triangulation. Returns the value supplied. With no argument, returns the
 current minimum angle constraint.
 
-Passing -1 to C<minimum_angle()> will cause the "q" switch to be ommited from
+Passing -1 to C<minimum_angle()> will cause the "q" switch to be omitted from
 the option string.
 
 =head2 doEdges, doVoronoi, doNeighbors
@@ -1163,9 +1187,9 @@ the "Q" switch is present. This module includes the "Q" switch by default, but
 you can override this by passing a false value to C<quiet()>.
 
 If you would like to see even more output regarding the triangulation process,
-there are are three levels of verbosity configuarable with repeated "V"
+there are are three levels of verbosity configurable with repeated "V"
 switches. Passing a number from 1 to 3 to the C<verbose()> method will enable 
-the correspoinding level of verbosity.
+the corresponding level of verbosity.
 
 =head1 METHODS TO ADD VERTICES AND SEGMENTS
 
@@ -1315,7 +1339,7 @@ The elements in the list have this structure:
 
 Returns a reference to a list of segments.
 
-    $segs  = $tri->segments(); # retrive the PSLG segments
+    $segs  = $tri->segments(); # retrieve the PSLG segments
 
 The segments in the list have this structure:
 
@@ -1325,7 +1349,7 @@ The segments in the list have this structure:
 
 Returns a reference to a list of edges.
 
-    $edges  = $tri->edges();    # retrive all the triangle edges
+    $edges  = $tri->edges();    # retrieve all the triangle edges
 
 The edges in the list have this structure:
 
@@ -1373,7 +1397,7 @@ at some point to hide them completely behind a less idiosyncratic interface.
 
 Michael E. Sheldrake, C<< <sheldrake at cpan.org> >>
 
-Triangle's author is Jonathan Richard Shewchuck
+Triangle's author is Jonathan Richard Shewchuk
 
 
 =head1 BUGS
@@ -1429,9 +1453,9 @@ or the Artistic License.
 
 See http://dev.perl.org/licenses/ for more information.
 
-=head2 Triangle licence
+=head2 Triangle license
 
-B<Triangle> by Jonathan Richard Shewchuck, copyright 2005, includes the following
+B<Triangle> by Jonathan Richard Shewchuk, copyright 2005, includes the following
 notice in the C source code. Please refer to the C source, included in with this
 Perl module distribution, for the full notice.
 
